/* */
/* This file calculates the alt azimuth correction for the hour axis */
/* */
/* 2003-SEP-12, Dr. Markus Wildi */

/*     Copyright (C) 2003 <Markus Wildi, Basel Switzerland> */

/*     This program is free software; you can redistribute it and/or modify */
/*     it under the terms of the GNU General Public License as published by */
/*     the Free Software Foundation; either version 2 of the License, or */
/*     any later version. */

/*     This program is distributed in the hope that it will be useful, */
/*     but WITHOUT ANY WARRANTY; without even the implied warranty of */
/*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the */
/*     GNU General Public License for more details. */

/*     You should have received a copy of the GNU General Public License */
/*     along with this program; if not, write to the Free Software */
/*     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

/* gcc -static -Xlinker -strip-all  -Wall ak.c  -lm -oak */
#include <stdio.h>
#include <string.h>
#include <math.h>


#define ALL 0 
#define RAMBAUT 10 
#define KING 20 
#define CHALLIS    30 
#define POLARIS    40 
#define A  130 
#define B1 140 
#define B2 150 

#define TRUE 1
#define FALSE 0


#define CONFIG_ENTRIES 19
#define RAMBAUT_ENTRIES 15
#define KINGA_ENTRIES 12
#define KINGB_ENTRIES 8
#define CHALLIS_ENTRIES 15
#define POLARIS_ENTRIES 10

#define TEMPERATURE 0
#define HEIGHT 1
#define OBS_LONGITUDE 2
#define OBS_LATITUDE 3
#define TZ_TIME 4
#define TZ_DATE 5
#define DECLINATION 6
#define RIGHTASCENSION 7
#define METHOD 8
#define VARIANT 9
#define RESOLUTION 10
#define DX 11
#define DY 12
#define DY1 13
#define DY2 14
#define DT 15
#define DT10 16
#define DT20 17
#define DECLINATION_M 18

#define ENTRIES_TZ 51


#define UTC  0
#define CET  1
#define CEST 2

#define POLARIS_RA 0.662403357
#define POLARIS_DEC 1.557952158

struct config {

  char   id[32] ;
  int    cnt ;
  double vl ;
  int    mthd_ramba ;
  int    mthd_kinga ;
  int    mthd_kingb ;
  int    mthd_chall ;
  int    mthd_pola ;
};
struct tz {

  char   id[32] ;
  double vl ;
};

double pressure_h2o(double temperature){

  double relative_hum= 0.5 ;
  double a= 0. ;
  double b= 0.;


  if( temperature > 273.15 ) {

    a=   7.5 ;
    b= 235.0 ;

  } else {

    a=   7.6 ;
    b= 240.7 ;
  }

  double value = relative_hum * 
  (6.107 * pow( 10, a* (temperature-273.15)/(b+ (temperature-273.15))))  ;

  return value ;
}

double pressure_tot(double height){

  double value = 1013.25 * pow(1. - (0.0065 * height / 288.),5.255) ;

  return value ;
}

double d_s( double pressure, double temperature) {

  double d_s ;

  d_s= pressure/temperature*  ( 1. + pressure * ( 57.9e-8 - 9.3250e-4/ temperature + 0.25844/pow(temperature, 2.))) ;
  return d_s ;
}
double d_w( double pressure, double temperature) {

  double d_w ;
 
  d_w= pressure/temperature * (  1. + pressure*( 1.+ 3.7e-4* pressure)* (-2.37321e-3+ 2.23366/temperature- 710.792/pow(temperature,  2.) 
	   + 7.75141e4/pow( temperature, 3.))) ;

  return d_w ;
}


double refractive_index( double lambda, double d_sv, double d_wv) {

  double index ;

  index= ( d_sv *( 2371.34 + 683939.7/( 130.- pow( lambda, -2.))  + 4547.3/(38.99- pow(lambda, -2.))) +
	   d_wv *( 6487.31 + 58.058 * pow( lambda, -2.) - 0.7115 * pow( lambda, -4.) + 0.08851* pow( lambda, -6.))) ;

  /* Attention NO +1 */
  return index * 1.e-8 + 1.  ;
}
double local_hour_angle( double year, double month, double day, double time, double alpha, double obs_longitude ){

  /* Meeus 2001 */


  double a=  280.46061837 ;
  double b=  360.98564736629 ;
  double c=  2451545.0  ;
  double d=  0.000387933 ;
  double e=  38710000 ;

  double AA= 0. ;
  double BB= 0. ;

  double JD     =   0 ;
  double T      = 0. ;
  double theta_0= 0. ;

  time= time /86400. ;

  if((month== 1) || (month== 2)) {
    month=month + 12 ;
    year= year -1 ;
  }

  AA= (int)( year/100) ;
  BB= (int)( 2 - AA + (int)( AA/4)) ;

  JD=   (int)(365.25  * ( year + 4716))
       + (int)(30.6001 * ( month + 1)) + day + time + BB  - 1524.5 ;

  printf("Julian Date    :    %12.6f\n", JD) ;

  T= (JD - 2451545.0)/36525. ;


  theta_0 = a + b * (JD- c) + d * pow(T, 2) - pow(T,3)/e ;


  theta_0 = fmod( theta_0 , 360.) ;
  if( theta_0 < 0) {

    theta_0=  360. +  theta_0 ; 
  }

  theta_0=  theta_0 /180. * M_PI ;

  double tau=  fmod(theta_0 - obs_longitude - alpha,  2. * M_PI) ;
  return tau ;
}

double refraction_dXp(double tau, double dec, double phi, double dtau, double n ){


  double value = -(n -1.) * cos( phi) * 
               (cos(dec) * cos( phi) + sin(dec) * sin( phi) * cos( tau+ dtau/2)) / 
  pow(( sin( phi) * sin( dec) + cos( phi) * cos( dec) * cos( tau+ dtau/2)),2) * dtau ;

  printf("Refraction  dX':  %12.2f arcsec\n", value *3600. * 180./ M_PI) ;

 return value ;
}

double refraction_dYp(double tau, double dec, double phi, double dtau, double n){


  double value = (n -1.) * 
              (cos( phi) * sin( phi) * sin( tau+ dtau/2)) / 
  pow(( sin( phi) * sin( dec) + cos( phi) * cos( dec) * cos( tau+ dtau/2)),2) * dtau ;

  printf("            dY':  %12.2f arcsec\n", value *3600. * 180./ M_PI) ;

  return value ;

}

double refraction_Yp(double tau, double dec, double phi, double n){


  double value = (n -1.) * 
              ( sin(phi) * cos(dec)- cos(phi) * sin(dec) * cos( tau)) / 
  (sin( phi) * sin( dec) + cos( phi) * cos( dec) * cos( tau)) ;


/*   printf("            RY':  %12.2f arcsec\n", value *3600. * 180./ M_PI) ; */

  return value ;

}




double units_radian(double radian_per_unit, double units){


  double value = units * radian_per_unit ;
  return value ;
}

double rambaut_A( double X, double Y, double tau, double dec, double phi, double dtau){


  double value= -( X/ sin( dec) * sin( tau +  dtau/2)  + Y * cos( tau + dtau/2))/cos( phi) / dtau ;

  return value ;}

double rambaut_k(double X, double Y, double tau, double dec, double phi, double dtau){


  double value= -( X/ sin( dec) * cos( tau + dtau/2) - Y * sin( tau + dtau/2)) / dtau ;

  return value ;
}

double King_A( double lambda_r, double h, double phi){



  double value= lambda_r * sin( h) / cos( phi) ;
  return value ;
}

double King_k(double lambda_r,double h){


  double value= lambda_r * cos( h) ;
  return value ;
}

double King_lambda_r(double X, double Y, double dtau){


  double value= sqrt( pow(X,2) + pow(Y,2))/ dtau ;
  return value ;
}

double King_h(double X, double Y){


  double value= -atan2( X, Y);

  return value ;
}


double dAA( double AA, double dec, double phi){


  double value= AA * sin( phi-dec) ;
  return value ;

}

double challis_denominator( double dtau1, double dtau2) {


  double value= 0. ;
  value= sin( -dtau1) - sin(-dtau2) + sin( dtau1 - dtau2) ;

/*   value= 2. * sin(.5) * sin( dtau2) * sin( .5 * ( dtau2- dtau1)) * sin( .5 * dtau1)  ;  */

  return value ;


}

double xy_plane_x(  double tau0, double dtau1, double dtau2, double Y1, double Y2) {

  double value= 0. ;

  value= ( (Y1 - Y2) * sin( tau0) + Y2 * sin( tau0 + dtau1) - Y1 * sin( tau0+ dtau2)) / challis_denominator( dtau1, dtau2) ;



/*   value= (Y2 * cos(.5 * (tau0+ dtau1 + tau0 )) * sin( .5 * ( dtau1)) - */
/*           Y1 * cos(.5 * (tau0+ dtau2 + tau0 )) * sin( .5 * ( dtau2))) / challis_denominator( dtau1, dtau2) ; */

  return value ;
}

double xy_plane_y(  double tau0, double dtau1, double dtau2, double Y1, double Y2) {


  double value= 0. ;

  value= ((-Y1 + Y2) * cos( tau0) - Y2 * cos( tau0 + dtau1) + Y1 * cos( tau0+ dtau2)) / challis_denominator( dtau1, dtau2) ;

/*   value= (Y2 * sin(.5 * (tau0+ dtau1 + tau0)) * sin( .5 * ( dtau1)) - */
/*           Y1 * sin(.5) * sin(.5 * (tau0+ dtau2 + tau0)) * sin( .5 * ( dtau2))) / challis_denominator( dtau1, dtau2) ; */

  return value ;
}



int main ( int argc, char *argv[]){


  FILE *fp ;

  int    line_n= 0 ;
  char   line[512] ;
  const char   *pline=  &line[0] ;
  int    number_arguments_from_file= 0 ;

  char   argument[512] ;
  const char   *pargument= &argument[0] ;
 
  char   identifier[100] ;
  const char   *pidentifier= &identifier[0] ;


  int error= 0 ;
  int variant_entries= 0 ;
  int actual_entries= 0 ;
  double threshold= 0. ;
  int kinga_entries= 0 ;
  int kingb_entries= 0 ;
  int challis_entries= 0 ;
  int polaris_entries= 0 ;
  int    ret ;
  int    i,j ;
  char   *ptoken ;
  char   *pos ;
  char    pound= (char) 35 ;
  char    minus= (char) 45 ;
  char sign_str[100] ;

  struct config cfg[CONFIG_ENTRIES] ={

    {"temperature"                    ,0 ,       -M_PI, TRUE,  FALSE, FALSE, TRUE, TRUE},
    {"height"                         ,0 ,       -M_PI, TRUE,  FALSE, FALSE, TRUE, TRUE},                
    {"obs_longitude"                  ,0 ,  -2. * M_PI, TRUE,  TRUE,  FALSE, TRUE, TRUE},             
    {"obs_latitude"                   ,0 ,  -2. * M_PI, TRUE,  TRUE,  TRUE,  TRUE, TRUE},          
    {"tz_time"                        ,0 ,  -2. * M_PI, TRUE,  TRUE,  FALSE, TRUE, TRUE},       
    {"tz_date"                        ,0 ,  -2. * M_PI, TRUE,  TRUE,  FALSE, TRUE, TRUE},         
    {"measurement_star_declination"   ,0 ,        M_PI, TRUE,  FALSE, FALSE, TRUE, FALSE},         
    {"measurement_star_rightascension",0 ,  -2. * M_PI, TRUE,  TRUE,  FALSE, TRUE, FALSE},
    {"method"                         ,0 ,  -3. * M_PI, TRUE,  TRUE,  TRUE,  TRUE, TRUE},              
    {"variant"                        ,0 ,  -3. * M_PI, TRUE,  TRUE,  TRUE,  TRUE, TRUE},              
    {"resolution"                     ,0 ,  -3. * M_PI, TRUE,  TRUE,  TRUE,  TRUE, TRUE},                
    {"dx"                             ,0 ,  -3. * M_PI, TRUE,  TRUE,  TRUE,  FALSE, FALSE},             
    {"dy"                             ,0 ,  -3. * M_PI, TRUE,  TRUE,  TRUE,  FALSE, FALSE},            
    {"dy1"                            ,0 ,  -3. * M_PI, FALSE, FALSE, FALSE, TRUE, FALSE},            
    {"dy2"                            ,0 ,  -3. * M_PI, FALSE, FALSE, FALSE, TRUE, FALSE},            
    {"dt"                             ,0 ,  -3. * M_PI, TRUE,  TRUE,  TRUE,  FALSE, FALSE},              
    {"dt10"                           ,0 ,  -3. * M_PI, FALSE, FALSE, FALSE, TRUE, FALSE},              
    {"dt20"                           ,0 ,  -3. * M_PI, FALSE, FALSE, FALSE, TRUE, FALSE},              
    {"correction_star_declination"    ,0 ,  -3. * M_PI, TRUE,  TRUE,  TRUE,  TRUE, TRUE}              
  };

  double vl[3] ;
  char tz_str[5] ;

  struct tz tz[ENTRIES_TZ] ={

            {"gmt"	,   0},		 // Greenwich Mean
	    {"ut"	,   0},		 // Universal (Coordinated)
	    {"utc"	,   0},
	    {"wet"	,   0},		 // Western European
	    {"wat"	,  -1.},	 // West Africa
	    {"at"	,  -2.},	 // Azores
	// For completeness.  BST is also British Summer}, and GST is also Guam Standard.
	//   {"bst"	,  -3.},	 // Brazil Standard
	//   {"gst"	,  -3.},	 // Greenland Standard
	//   {"nft"	,  -3.-1800},// Newfoundland
	//   {"nst"	,  -3.-1800},// Newfoundland Standard
	    {"ast"	,  -4.},	 // Atlantic Standard
	    {"est"	,  -5.},	 // Eastern Standard
	    {"cst"	,  -6.},	 // Central Standard
	    {"cest"	,  +2.},	 // Central European Summer
	    {"mst"	,  -7.},	 // Mountain Standard
	    {"pst"	,  -8.},	 // Pacific Standard
	    {"yst"	,  -9.},	 // Yukon Standard
	    {"hst"	, -10.},	 // Hawaii Standard
	    {"cat"	, -10.},	 // Central Alaska
	    {"ahst"	, -10.},	 // Alaska-Hawaii Standard
	    {"nt"	, -11.},	 // Nome
	    {"idlw"	, -12.},	 // International Date Line West
	    {"cet"	,  +1.},	 // Central European
	    {"met"	,  +1.},	 // Middle European
	    {"mewt"	,  +1.},	 // Middle European Winter
	    {"swt"	,  +1.},	 // Swedish Winter
	    {"fwt"	,  +1.},	 // French Winter
	    {"eet"	,  +2.},	 // Eastern Europe}, USSR Zone 1
	    {"bt"	,  +3.},	 // Baghdad}, USSR Zone 2
	//   {"it"	,  +3.+1800},// Iran
	    {"zp4"	,  +4.},	 // USSR Zone 3
	    {"zp5"	,  +5.},	 // USSR Zone 4
	    {"ist"	,  +5.5},// Indian Standard
	    {"zp6"	,  +6.},	 // USSR Zone 5
	// For completeness.  NST is also Newfoundland Stanard}, and SST is also Swedish Summer.
	//   {"nst"	,  +6.+1800},// North Sumatra
	//   {"sst"	,  +7.},	 // South Sumatra}, USSR Zone 6
	    {"wast"	,  +7.},	 // West Australian Standard
	//   {"jt"	,  +7.+1800},// Java (3pm in Cronusland!)
	    {"cct"	,  +8.},	 // China Coast}, USSR Zone 7
	    {"jst"	,  +9.},	 // Japan Standard}, USSR Zone 8
	//   {"cast"	,  +9.+1800},// Central Australian Standard
	    {"east"	, +10.},	 // Eastern Australian Standard
	    {"gst"	, +10.},	 // Guam Standard}, USSR Zone 9
	    {"nzt"	, +12.},	 // New Zealand
	    {"nzst"	, +12.},	 // New Zealand Standard
	    {"idle"	, +12.},	 // International Date Line East
	//   {"ndt"  ,	-1.5},	 // Newfoundland Daylight
	    {"adt"  ,	-3.},	 // Atlantic Daylight
	    {"edt"  ,	-4.},	 // Eastern Daylight
	    {"cdt"  ,	-5.},	 // Central Daylight
	    {"mdt"  ,	-6.},	 // Mountain Daylight
	    {"pdt"  ,	-7.},	 // Pacific Daylight
	    {"ydt"  ,	-8.},	 // Yukon Daylight
	    {"hdt"  ,	-9.},	 // Hawaii Daylight
	    {"bst"  ,	+1.},	 // British Summer
	    {"mest" ,	+2.},	 // Middle European Summer
	    {"sst"  ,	+2.},	 // Swedish Summer
	    {"fst"  ,	+2.},	 // French Summer
	    {"wadt" ,	+8.},	 // West Australian Daylight
	//   {"cadt" ,  +10.5},	 // Central Australian Daylight
	    {"eadt" ,  +11.},	 // Eastern Australian Daylight
	    {"nzdt" ,  +13.},	 // New Zealand Daylight
 } ;

  double time_utc= 0. ;
  double time_local= 0. ;

  double year =0. ; 
  double month=0. ; 
  double day  =0. ; 

  double dtau= 0. ;
  double dtau1= 0. ;
  double dtau2= 0. ;
  double tau = 0. ;
  double tau0 = 0. ;
  const double omega_sid= 2. *  M_PI / 86164.2 ;


  double pressure_water= 0.;
  double pressure_total= 0.;
  double airpressure= 0.;

  double d_sv=0.;
  double d_wv=0.;
  double n=0.;
  const double lambda = .55 ;

  double dXp = 0. ;
  double dYp = 0. ;
  double X   = 0. ;
  double Y   = 0. ;
  double Xp   = 0. ;
  double Yp   = 0. ;

  double RY0p= 0. ;
  double RY1p= 0. ;
  double RY2p= 0. ;
  double Y1   = 0. ;
  double Y2   = 0. ;
  double Y1p   = 0. ;
  double Y2p   = 0. ;

  double x= 0. ;
  double y= 0. ;


  double AA= 0. ;
  double kk= 0. ;

  double lambda_r= 0. ;
  double h= 0. ;
  double challis_lambda= 0. ;
  double challis_h= 0. ;
  double polaris_lambda= 0. ;
  double polaris_h= 0. ;








  printf("\nThis the hour axis position calculator V 0.1\n") ;
  printf("-----------------------------------------------------\n\n") ;
  printf("This program is based on the book\n\n") ;
  printf("Forms of images in stellar photography, the companion\n") ;
  printf("      Prime focus astrophotography at the lower limit\n") ;
  printf("\nby Markus Wildi and Arthur von Kaenel\n") ;
  printf("markus.wildi@datacomm.ch\n") ;
  printf("-----------------------------------------------------\n\n") ;



  if( argc > 1) {
    if(( fp= fopen( argv[1], "r"))==NULL) {

	printf( "Error opening configuration file %s\n",  argv[1]) ;
	return 1 ;
      
    }
  } else {

    printf( "A configuration file name is needed, trying to open stdin\n\n") ;
    //    return(1) ;
    fp= stdin ;
  }

 NEXT:    while( fgets( line, 512, fp))  {

   line_n++ ;

   ptoken= strchr( pline, pound) ;    

   if( ptoken==  pline ) {

     goto NEXT ;   
   }

   identifier[0]= '\0' ;

   ptoken= strtok( line , "#") ;    
 
   pidentifier= strtok( line , "=") ;    

   pos= strchr( pidentifier, (char) 32) ;    
 
   if( pidentifier==  pos) {

     goto NEXT ;  
   }

   if(pos != NULL) {
     strcpy( pos, "\0") ;
   } 

   pargument= strtok( NULL , " \n") ;    

   if( pargument==NULL) {

     printf("Error on line %d, identifier %s missing argument\n", line_n, pidentifier) ;
     return 1 ;
   }

  for( i=0 ; i < CONFIG_ENTRIES; i++) {

     if(( ret=strcmp(pidentifier, cfg[i].id ))== 0) {

       if( i== METHOD) {

	 if(( ret=strcmp(pargument, "Rambaut")) ==0) {
	   
	   cfg[i].vl= RAMBAUT ;
           
	 } else if(( ret=strcmp(pargument, "King")) ==0) {

	   cfg[i].vl= KING ;

	   kinga_entries++ ;
	   kingb_entries++ ;
	 } else if(( ret=strcmp(pargument, "Challis")) ==0) {

	   cfg[i].vl= CHALLIS ;
	   challis_entries++ ;
	 } else if(( ret=strcmp(pargument, "Polaris")) ==0) {

	   cfg[i].vl= POLARIS ;
	   polaris_entries++ ;
	 } else {

	   printf("Error on line %d, identifier %s, argument %s\n", line_n, pidentifier, pargument) ;
	   return(1) ;  
	 }
         break ;

       } else if( i== VARIANT) {
	 if( cfg[METHOD].vl== RAMBAUT) {

	   if(( ret=strcmp(pargument, "A")) ==0) {

	     cfg[i].vl= A ;
	     break ;
	   } else {

	     printf("Error on line %d, identifier %s, argument %s\n", line_n, pidentifier, pargument) ;
	     return(1) ;  
	   }
	 } else if( cfg[METHOD].vl== KING) {

	   if(( ret=strcmp(pargument, "A")) ==0) {

	     cfg[i].vl= A ;
	     kinga_entries++ ;
	     break ;
	   }else if(( ret=strcmp(pargument, "B1")) ==0) {

	     cfg[i].vl= B1 ;
	     kingb_entries++ ;
	     break ;
	   }else if(( ret=strcmp(pargument, "B2")) ==0) {

	     cfg[i].vl= B2 ;
	     kingb_entries++ ;
	     break ;
	   } else {

	     printf("Error on line %d, identifier %s, argument %s\n", line_n, pidentifier, pargument) ;
	     return(1) ;  
	   }
	 } else if( cfg[METHOD].vl== CHALLIS) {

	   if(( ret=strcmp(pargument, "A")) ==0) {

	     cfg[i].vl= A ;
	     challis_entries++ ;
	     break ;
	   } else {

	    printf("Error on line %d, identifier %s, argument %s\n", line_n, pidentifier, pargument) ;
	    return(1) ;  
	   }
	 } else if( cfg[METHOD].vl== POLARIS) {

	   if(( ret=strcmp(pargument, "A")) ==0) {

	     cfg[i].vl= A ;
	     polaris_entries++ ;
	     break ;
	   } else {

	    printf("Error on line %d, identifier %s, argument %s\n", line_n, pidentifier, pargument) ;
	    return(1) ;  
	   }
	 } else {

	   printf("Error on line %d, argument %s\n", line_n, pargument) ;
	   return(1) ;  
	 }
         break ;

       } else if( i== TZ_TIME) {

	 if((ret=sscanf( pargument, "%lf:%lf:%lf:%s", &vl[0], &vl[1], &vl[2],tz_str))== 4) {

	   for( j=0 ; j < ENTRIES_TZ; j++) {
	     if(( ret=strcmp(tz_str, tz[j].id ))== 0) {

	       cfg[TZ_TIME].vl = fabs(vl[0]) * 3600. + vl[1] * 60. +  vl[2] ;
	       ret=sscanf( pargument, "%s:%lf:%lf", &sign_str[0], &vl[1], &vl[2]) ;
	       if((strchr( pargument, minus)) != NULL ) {

		 cfg[TZ_TIME].vl= - cfg[TZ_TIME].vl;
	       }
	       time_utc=  cfg[TZ_TIME].vl - tz[j].vl * 3600.;
	       break ;
	     }
	   }
	   if( j== ENTRIES_TZ) {

	     printf("Error on line %d, undefinded time zone %s\nIt is one of these:\n\n", line_n, tz_str) ;

	     for( j=0 ; j < ENTRIES_TZ; j++) {
	       printf("Timezone: %5s offset: %+5.1f\n", tz[j].id, tz[j].vl) ;
	     }
	     return(1) ;
	   }
	   kinga_entries++ ;
	   challis_entries++ ;
	   polaris_entries++ ;
	 } else {

	    printf("Error on line %d, identifier %s, argument %s\n", line_n, pidentifier, pargument) ;
	   return(1) ;
	 }    

         break ;
       } else if( i== DECLINATION_M) {

	 if((ret=sscanf( pargument, "%lf:%lf:%lf", &vl[0], &vl[1], &vl[2]))== 3) {

	   cfg[DECLINATION_M].vl=  (fabs(vl[0]) + vl[1]/60. +  vl[2]/3600)/ 180. * M_PI ;

	   ret=sscanf( pargument, "%s:%lf:%lf", &sign_str[0], &vl[1], &vl[2]) ;
	   if((strchr( pargument, minus)) != NULL ) {

	     cfg[DECLINATION_M].vl= - cfg[DECLINATION_M].vl;
	   }


	 } else if((ret=sscanf( pargument, "%lf", &vl[0]))== 1) {

	   cfg[DECLINATION_M].vl=  vl[0]/ 180. * M_PI ;

	 } else {

	    printf("Error on line %d, identifier %s, argument %s\n", line_n, pidentifier, pargument) ;
	   return(1) ;
	 }
	 kinga_entries++ ;
	 kingb_entries++ ;
	 challis_entries++ ;
	 polaris_entries++ ;
         break ;
       } else if( i== DECLINATION) {

	 if((ret=sscanf( pargument, "%lf:%lf:%lf", &vl[0], &vl[1], &vl[2]))== 3) {

	   cfg[DECLINATION].vl=  (fabs(vl[0]) + vl[1]/60. +  vl[2]/3600)/ 180. * M_PI ;

	   ret=sscanf( pargument, "%s:%lf:%lf", &sign_str[0], &vl[1], &vl[2]) ;
	   if((strchr( pargument, minus)) != NULL ) {

	     cfg[DECLINATION].vl= - cfg[DECLINATION].vl;
	   }

	 } else if((ret=sscanf( pargument, "%lf", &vl[0]))== 1) {

	   cfg[DECLINATION].vl=  vl[0]/ 180. * M_PI ;

	 } else {

	    printf("Error on line %d, identifier %s, argument %s\n", line_n, pidentifier, pargument) ;
	   return(1) ;
	 }
	 challis_entries++ ;
         break ;
       } else if( i== RIGHTASCENSION) {

	 if((ret=sscanf( pargument, "%lf:%lf:%lf", &vl[0], &vl[1], &vl[2]))== 3) {

	   cfg[RIGHTASCENSION].vl=  (fabs(vl[0]) + vl[1]/60. +  vl[2]/3600.) *15./180. * M_PI ;

	   ret=sscanf( pargument, "%s:%lf:%lf", &sign_str[0], &vl[1], &vl[2]) ;
	   if((strchr( pargument, minus)) != NULL ) {

	     cfg[RIGHTASCENSION].vl= - cfg[RIGHTASCENSION].vl;
	   }

	 } else if((ret=sscanf( pargument, "%lf", &vl[0]))== 1) {

	   cfg[RIGHTASCENSION].vl=  vl[0]/ 180. * M_PI ;
	 } else {

	    printf("Error on line %d, identifier %s, argument %s\n", line_n, pidentifier, pargument) ;
	   return(1) ;
	 }

	 kinga_entries++ ;
	 challis_entries++ ;
         break ;
       } else if( i== OBS_LATITUDE) {

	 if((ret=sscanf( pargument, "%lf:%lf:%lf", &vl[0], &vl[1], &vl[2]))== 3) {

	   cfg[OBS_LATITUDE].vl=  (fabs(vl[0]) + vl[1]/60. +  vl[2]/3600.)/ 180. * M_PI ;

	   ret=sscanf( pargument, "%s:%lf:%lf", &sign_str[0], &vl[1], &vl[2]) ;
	   if((strchr( pargument, minus)) != NULL ) {

	     cfg[OBS_LATITUDE].vl= - cfg[OBS_LATITUDE].vl;
	   }
	 } else if((ret=sscanf( pargument, "%lf", &vl[0]))== 1) {

	   cfg[OBS_LATITUDE].vl=  vl[0]/ 180. * M_PI ;

	 } else {

	    printf("Error on line %d, identifier %s, argument %s\n", line_n, pidentifier, pargument) ;
	   return(1) ;
	 }
	 kinga_entries++ ;
	 kingb_entries++ ;
	 challis_entries++ ;
	 polaris_entries++ ;
         break ;
       } else if( i== OBS_LONGITUDE) {

	 if((ret=sscanf( pargument, "%lf:%lf:%lf", &vl[0], &vl[1], &vl[2]))== 3) {


	   cfg[OBS_LONGITUDE].vl=  (fabs(vl[0]) + vl[1]/60. +  vl[2]/3600.)/180. * M_PI ;

	   ret=sscanf( pargument, "%s:%lf:%lf", &sign_str[0], &vl[1], &vl[2]) ;
	   if((strchr( pargument, minus)) != NULL ) {

	     cfg[OBS_LONGITUDE].vl= - cfg[OBS_LONGITUDE].vl;
	   }

	 } else if((ret=sscanf( pargument, "%lf", &vl[0]))== 1) {

	   cfg[OBS_LONGITUDE].vl=  vl[0]/ 180. * M_PI ;
	 } else {

	    printf("Error on line %d, identifier %s, argument %s\n", line_n, pidentifier, pargument) ;
	   return(1) ;
	 }
	 kinga_entries++ ;
	 challis_entries++ ;
	 polaris_entries++ ;
         break ;

       } else if( i== DT) {

	 
	 if(( ret=sscanf(pargument, "%lf", &cfg[i].vl)) !=1) {

	    printf("Error on line %d, identifier %s, argument %s\n", line_n, pidentifier, pargument) ;
	   return(1) ;
	 }
         dtau= omega_sid *  cfg[i].vl ;
	 kinga_entries++ ;
	 kingb_entries++ ;
         break ;
       } else if( i== DT10) {

	 
	 if(( ret=sscanf(pargument, "%lf", &cfg[i].vl)) !=1) {

	    printf("Error on line %d, identifier %s, argument %s\n", line_n, pidentifier, pargument) ;
	   return(1) ;
	 }
         dtau1= omega_sid *  cfg[i].vl ;
	 challis_entries++ ;
         break ;
       } else if( i== DT20) {

	 
	 if(( ret=sscanf(pargument, "%lf", &cfg[i].vl)) !=1) {

	    printf("Error on line %d, identifier %s, argument %s\n", line_n, pidentifier, pargument) ;
	   return(1) ;
	 }
         dtau2= omega_sid *  cfg[i].vl ;
	 challis_entries++ ;
         break ;
       } else if( i== TEMPERATURE) {
	 
	 if(( ret=sscanf(pargument, "%lf", &cfg[i].vl)) !=1) {

	    printf("Error on line %d, identifier %s, argument %s\n", line_n, pidentifier, pargument) ;
	   return(1) ;
	 }
         cfg[i].vl = cfg[i].vl + 273.15;
         break ;

       } else if( i== TZ_DATE) {

	 if((ret=sscanf( pargument, "%lf:%lf:%lf", &year, &month, &day))== 3) {

	   cfg[i].vl= 0. ;
	 } else {

	   printf("Error on line %d, identifier %s, argument %s\n", line_n, pidentifier, pargument) ;
	   return(1) ;
	 }
	 kinga_entries++ ;
	 challis_entries++ ;
	 polaris_entries++ ;
         break ;
       } else {

	 if(( ret=sscanf(pargument, "%lf", &cfg[i].vl)) !=1) {

	   printf("Error on line %d, identifier %s, argument %s\n", line_n, pidentifier, pargument) ;
	   return(1) ;
	 }
	 kinga_entries++ ;
	 kingb_entries++ ;
	 challis_entries++ ;
	 polaris_entries++ ;
         break ;
       }
     }
   }


   if( i== CONFIG_ENTRIES) {

     printf("Error on line %d, identifier %s\n", line_n, pidentifier) ;

     printf( "For each method, set the following parameters\n\n") ;

     printf( "Rambaut A\n") ;

     for( i=0 ; i < CONFIG_ENTRIES; i++)  {
       if( cfg[i].mthd_ramba== TRUE) {
	 printf( "Identifier: %32s\n", cfg[i].id) ;
       }
     }
     printf( "\nKing A\n") ;

     for( i=0 ; i < CONFIG_ENTRIES; i++)  {
       if( cfg[i].mthd_kinga== TRUE) {
	 printf( "Identifier: %32s\n", cfg[i].id) ;
       }
     }
     printf( "\nKing B1, B2\n") ;

     for( i=0 ; i < CONFIG_ENTRIES; i++)  {
       if( cfg[i].mthd_kingb== TRUE) {
	 printf( "Identifier: %32s\n", cfg[i].id) ;
       }
     }
     printf( "\nChallis A\n") ;

     for( i=0 ; i < CONFIG_ENTRIES; i++)  {
       if( cfg[i].mthd_chall== TRUE) {
	 printf( "Identifier: %32s\n", cfg[i].id) ;
       }
     }
     printf( "\nPolaris A\n") ;

     for( i=0 ; i < CONFIG_ENTRIES; i++)  {
       if( cfg[i].mthd_pola== TRUE) {
	 printf( "Identifier: %32s\n", cfg[i].id) ;
       }
     }

     return(1) ;

   } else {
     cfg[i].cnt++ ;
     number_arguments_from_file++ ; 
   }
 }
  
 if( cfg[METHOD].vl== RAMBAUT) {

    if( number_arguments_from_file < RAMBAUT_ENTRIES) {
    
      printf("Not all parameters set in file %s\n", argv[1]) ;
      printf("All these must be set in addition:\n\n") ;
      for( i=0 ; i < CONFIG_ENTRIES; i++) {


	if((  cfg[i].vl <= -M_PI) && ( cfg[i].mthd_ramba== TRUE)) { 
	  printf( "Identifier: %32s\n", cfg[i].id) ;
	  error++ ;
	}
      }
    } else {

      for( i=0 ; i < CONFIG_ENTRIES; i++) {
	if((  cfg[i].cnt > 1) && ( cfg[i].mthd_ramba== TRUE)){ 
	  printf( "Identifier st multiple times: %32s\n", cfg[i].id) ;
	  error++ ;
	}
      }
    }
  } else if( cfg[METHOD].vl== KING){

    if( cfg[VARIANT].vl== A){

      variant_entries= KINGA_ENTRIES;
      actual_entries= kinga_entries ;
      threshold= -2. *M_PI ;

      if( actual_entries< variant_entries) {
    
	printf("Not all parameters set in file %s\n", argv[1]) ;
	printf("All these must be set in addition:\n\n") ;

	for( i=0 ; i < CONFIG_ENTRIES; i++) {

	  if((  cfg[i].vl <= threshold)  && ( cfg[i].mthd_kinga== TRUE)){ 
	    printf( "Identifier: %32s\n", cfg[i].id) ;
	    error++ ;
	  }
	}
      } else {
	
	for( i=0 ; i < CONFIG_ENTRIES; i++) {
	  if((  cfg[i].cnt > 1) && ( cfg[i].mthd_kinga== TRUE)){ 
	    printf( "Identifier st multiple times: %32s\n", cfg[i].id) ;
	    error++ ;
	  }
	}
      }
    } else if( cfg[VARIANT].vl== B1) {

      variant_entries= KINGB_ENTRIES;
      actual_entries= kingb_entries ;
      threshold= -3. *M_PI ;

      if( actual_entries< variant_entries) {
    
	printf("Not all parameters set in file %s\n", argv[1]) ;
	printf("All these must be set in addition:\n\n") ;

	for( i=0 ; i < CONFIG_ENTRIES; i++) {

	  if((  cfg[i].vl <= threshold)  && ( cfg[i].mthd_kingb== TRUE)){ 
	    printf( "Identifier: %32s\n", cfg[i].id) ;
	    error++ ;
	  }
	}
      } else {
	
	for( i=0 ; i < CONFIG_ENTRIES; i++) {
	  if((  cfg[i].cnt > 1) && ( cfg[i].mthd_kingb== TRUE)){ 
	    printf( "Identifier st multiple times: %32s\n", cfg[i].id) ;
	    error++ ;
	  }
	}
      }
    } else if( cfg[VARIANT].vl== B2) {

      variant_entries= KINGB_ENTRIES;
      actual_entries= kingb_entries ;
      threshold= -3. *M_PI ;
      if( actual_entries< variant_entries) {
    
	printf("Not all parameters set in file %s %d\n", argv[1], actual_entries) ;
	printf("All these must be set in addition:\n\n") ;

	for( i=0 ; i < CONFIG_ENTRIES; i++) {

	  if((  cfg[i].vl <= threshold)  && ( cfg[i].mthd_kingb== TRUE)){ 
	    printf( "Identifier: %32s\n", cfg[i].id) ;
	    error++ ;
	  }
	}
      } else {
	
	for( i=0 ; i < CONFIG_ENTRIES; i++) {
	  if((  cfg[i].cnt > 1) && ( cfg[i].mthd_kingb== TRUE)){ 
	    printf( "Identifier st multiple times: %32s\n", cfg[i].id) ;
	    error++ ;
	  }
	}
      }
    }
 
  } else if( cfg[METHOD].vl== CHALLIS) {
 
    if( challis_entries < CHALLIS_ENTRIES) {
    
      printf("Not all parameters set in file %s\n", argv[1]) ;
      printf("All these must be set in addition:\n\n") ;
      for( i=0 ; i < CONFIG_ENTRIES; i++) {


	if((  cfg[i].vl <= -M_PI)   && ( cfg[i].mthd_chall== TRUE)){ 
	  printf( "Identifier: %32s\n", cfg[i].id) ;
	  error++ ;
	}
      }
    } else {

      for( i=0 ; i < CONFIG_ENTRIES; i++) {
	if((  cfg[i].cnt > 1) && ( cfg[i].mthd_chall== TRUE)){ 
	  printf( "Identifier st multiple times: %32s\n", cfg[i].id) ;
	  error++ ;
	}
      }
    }
  } else if( cfg[METHOD].vl== POLARIS) {
 
    if( polaris_entries < POLARIS_ENTRIES) {
    
      for( i=0 ; i < CONFIG_ENTRIES; i++) {

	if((  cfg[i].vl <= -M_PI)   && ( cfg[i].mthd_pola== TRUE)){ 
	  printf( "Identifier: %32s\n", cfg[i].id) ;
	  error++ ;
	}
      }
    } else {

      for( i=0 ; i < CONFIG_ENTRIES; i++) {
	if((  cfg[i].cnt > 1) && ( cfg[i].mthd_pola== TRUE)){ 
	  printf( "Identifier st multiple times: %32s\n", cfg[i].id) ;
	  error++ ;
	}
      }
    }
  }

  if( error> 0) {

    printf("\n\nUnits and formats:\n\n") ;
    printf("Temeprature : Degree Celsius \n") ;
    printf("Height      : meter\n") ;
    printf("Angles      : decimal degree or sexagesimal\n") ;
    printf("Time        : hh:mm:dd.d:timezone\n") ;
    printf("Date        : YYYY:MM:DD\n") ;
    printf("Method      : Rambaut, King or Challis\n") ;
    printf("Variant     : A (Rambaut, Challis)\n") ;
    printf("Variant     : A, B1, B2 (King)\n") ;
    printf("Resolution  : arcsec/unit\n") ;
    printf("dx          : your units\n") ;
    printf("dy, dy1, dy2: your units\n") ;
    printf("dt          : Seconds\n") ;


    printf("\nFurther help:\n\n   Forms of images in stellar photography, the companion\n") ;
    printf("         Prime focus astrophotography at the lower limit\n") ;
    printf("\nmarkus.wildi@datacomm.ch\n\n") ;

    return(1) ;
  }
  /* local time */


  time_local= time_utc + cfg[OBS_LONGITUDE].vl * 86400. /(2. * M_PI) ;


  if( cfg[METHOD].vl== RAMBAUT) {

/*     for( i=0 ; i < ENTRIES; i++) { */

/*       printf( "Identifier: %32s, %f\n", cfg[i].id, cfg[i].vl) ; */
	
/*     } */
    
    pressure_water= pressure_h2o( cfg[TEMPERATURE].vl) ;
    pressure_total= pressure_tot( cfg[HEIGHT].vl) ;
    airpressure   = pressure_total- pressure_water ;

    printf( "Pressure total :  %12.2f hPa\n", pressure_total) ;
    printf( "Dry air        :  %12.2f hPa\n", airpressure) ;
    printf( "Water          :  %12.2f hPa\n", pressure_water) ;

    d_sv= d_s( airpressure, cfg[TEMPERATURE].vl) ;
    d_wv= d_w( pressure_water, cfg[TEMPERATURE].vl) ;

    n= refractive_index( lambda, d_sv, d_wv) ;

    printf( "Refractive Ind.:         %12.9f\n", n) ;

    tau= local_hour_angle(year, month, day, time_utc, cfg[RIGHTASCENSION].vl, cfg[OBS_LONGITUDE].vl) ;

    printf( "UTC time       :  %12.2f seconds\n", time_utc) ;
    printf( "Local hourangle:  %12.2f degree\n", tau/M_PI * 180.) ;

    printf( "dtau           :  %15.5f rad\n", dtau) ;

    dXp =  refraction_dXp( tau, cfg[DECLINATION].vl, cfg[OBS_LATITUDE].vl, dtau, n) ;
    dYp =  refraction_dYp( tau, cfg[DECLINATION].vl, cfg[OBS_LATITUDE].vl, dtau, n) ;

    Xp= units_radian(cfg[RESOLUTION].vl/3600./180. * M_PI, cfg[DX].vl)  ;
    Yp= units_radian(cfg[RESOLUTION].vl/3600./180. * M_PI, cfg[DY].vl)  ;

    X= Xp+  dXp ;
    Y= Yp+  dYp ;

    printf("Measured     X':  %12.2f arcsec\n",  Xp* 180. * 3600./M_PI) ;
    printf("             Y':  %12.2f arcsec\n",  Yp* 180. * 3600./M_PI) ;
   
    printf("Rambaut      X :  %12.2f arcsec\n", X * 180. * 3600./M_PI) ;
    printf("             Y :  %12.2f arcsec\n", Y * 180. * 3600./M_PI) ;

    AA= rambaut_A( X, Y, tau, cfg[DECLINATION].vl, cfg[OBS_LATITUDE].vl, dtau) ;
    kk= rambaut_k( X, Y, tau, cfg[DECLINATION].vl, cfg[OBS_LATITUDE].vl, dtau) ;

    printf("\nRambaut A      :  %12.2f arcsec\n", AA * 180. * 3600./M_PI) ;
    printf("Rambaut k      :  %12.2f arcsec\n", kk * 180. * 3600./M_PI) ;

    double rambaut_h= atan2( AA * cos(cfg[OBS_LATITUDE].vl), kk) ;

    if( rambaut_h< 0. ) {

      rambaut_h= 2. * M_PI + rambaut_h ;
    }
    printf("Rambaut h      :  %12.2f degree\n", rambaut_h * 180. /M_PI) ;

    printf("Rambaut lambda :  %12.2f arcsec\n", sqrt( pow( AA * cos(cfg[OBS_LATITUDE].vl), 2) + pow(kk,2)) * 180. * 3600. /M_PI) ;
    //
  } else if( cfg[METHOD].vl== KING) {

    printf( "dtau           :  %15.5f rad\n", dtau) ;


    X= units_radian(cfg[RESOLUTION].vl/3600./180. * M_PI, cfg[DX].vl)  ;
    Y= units_radian(cfg[RESOLUTION].vl/3600./180. * M_PI, cfg[DY].vl)  ;

    lambda_r= King_lambda_r(X, Y, dtau) ;

    if( cfg[VARIANT].vl== A) {

      tau= local_hour_angle(year, month, day, time_utc, cfg[RIGHTASCENSION].vl, cfg[OBS_LONGITUDE].vl) ;
      printf( "Local hourangle:  %12.2f degree measurement star\n", tau/M_PI * 180.) ;

      h= King_h(-X, -Y) ;
      h= h + dtau/2. + tau ;

    } else if( cfg[VARIANT].vl== B1) {

      h= King_h( X, Y) ;
      h= h + dtau/2. ;
    } else if( cfg[VARIANT].vl== B2) {

      h= King_h( X, Y) ;
    }
    if( h < 0.) {

      h= 2. * M_PI +  h  ;
    }

    h= fmod( h,  2. * M_PI ) ;
 
    printf("\nKing h         :  %12.2f degree\n",  h* 180. /M_PI) ;
    printf("King lambda    :  %12.2f arcsec\n", lambda_r* 180. * 3600./M_PI) ;
    printf("King lambda    :  %12.2f deg\n", lambda_r* 180. /M_PI) ;
    
    AA= King_A( lambda_r, h, cfg[OBS_LATITUDE].vl) ;
    kk= King_k( lambda_r, h) ;

    printf("\nKing A         :  %12.2f arcsec\n",  AA * 3600. * 180. /M_PI) ;
    printf("King k         :  %12.2f arcsec\n",  kk* 180. * 3600./M_PI) ;

  } else if( cfg[METHOD].vl== CHALLIS) {

/*     for( i=0 ; i < CONFIG_ENTRIES; i++) { */

/*  	if(( cfg[i].mthd_chall== TRUE)) {  */
/* 	  printf( "Identifier: %32s, %f\n", cfg[i].id, cfg[i].vl) ; */
/* 	}	 */
/*     } */
    printf( "\n") ;

    pressure_water= pressure_h2o( cfg[TEMPERATURE].vl) ;
    pressure_total= pressure_tot( cfg[HEIGHT].vl) ;
    airpressure   = pressure_total- pressure_water ;

    printf( "Pressure total :  %12.2f hPa\n", pressure_total) ;
    printf( "Dry air        :  %12.2f hPa\n", airpressure) ;
    printf( "Water          :  %12.2f hPa\n", pressure_water) ;

    d_sv= d_s( airpressure, cfg[TEMPERATURE].vl) ;
    d_wv= d_w( pressure_water, cfg[TEMPERATURE].vl) ;

    n= refractive_index( lambda, d_sv, d_wv) ;


    printf( "Refractive Ind.:         %12.9f\n", n) ;

    tau0= local_hour_angle(year, month, day, time_utc, cfg[RIGHTASCENSION].vl, cfg[OBS_LONGITUDE].vl) ;

    printf( "UTC time       :  %12.2f seconds\n", time_utc) ;
    printf( "Local hourangle:  %12.2f degree\n", tau0/M_PI * 180.) ;

/*     printf( "dtau1          :  %15.5f rad\n", dtau1) ; */
/*     printf( "dtau2          :  %15.5f rad\n", dtau2) ; */

    RY0p =  refraction_Yp( tau0,         cfg[DECLINATION].vl, cfg[OBS_LATITUDE].vl, n) ;
    RY1p =  refraction_Yp( tau0 + dtau1, cfg[DECLINATION].vl, cfg[OBS_LATITUDE].vl, n) ;
    RY2p =  refraction_Yp( tau0 + dtau2, cfg[DECLINATION].vl, cfg[OBS_LATITUDE].vl, n) ;

    Y1p= units_radian(cfg[RESOLUTION].vl/3600./180. * M_PI, cfg[DY1].vl)  ;
    Y2p= units_radian(cfg[RESOLUTION].vl/3600./180. * M_PI, cfg[DY2].vl)  ;

    Y1= Y1p-  (RY1p - RY0p) ;
    Y2= Y2p-  (RY2p - RY0p) ;


    printf("Refraction  RY0p':  %12.2f arcsec\n",  RY0p* 180. * 3600./M_PI) ;
    printf("            RY1p':  %12.2f arcsec\n",  RY1p* 180. * 3600./M_PI) ;
    printf("            RY2p':  %12.2f arcsec\n",  RY2p* 180. * 3600./M_PI) ;


    printf("Measured    Y1':  %12.2f arcsec\n",  Y1p* 180. * 3600./M_PI) ;
    printf("            Y2':  %12.2f arcsec\n",  Y2p* 180. * 3600./M_PI) ;
   
    printf("Challis     Y1 :  %12.2f arcsec\n", Y1 * 180. * 3600./M_PI) ;
    printf("            Y2 :  %12.2f arcsec\n", Y2 * 180. * 3600./M_PI) ;


    x= xy_plane_x( tau0, dtau1, dtau2, Y1, Y2) ;
    y= xy_plane_y( tau0, dtau1, dtau2, Y1, Y2) ;

/*     printf("\nChallis x      :  %12.2f arcsec\n", x * 180. * 3600./M_PI) ; */
/*     printf("Challis y      :  %12.2f arcsec\n", y * 180. * 3600./M_PI) ; */

    challis_lambda= sqrt( pow(x,2) + pow(y,2)) ;

    printf("Challis lambda :  %12.2f arcsec\n", challis_lambda * 180. * 3600./M_PI) ;



    challis_h= atan2( y, x) ; // that's correct this way
    if( challis_h < 0) {

      challis_h = 2. * M_PI + challis_h ;
    }

    printf("Challis h      :  %12.2f deg\n", challis_h * 180./M_PI) ;

    AA= King_A( challis_lambda, challis_h, cfg[OBS_LATITUDE].vl) ;
    kk= King_k( challis_lambda, challis_h) ;

    printf("\nChallis A      :  %12.2f arcsec\n",  AA * 3600. * 180. /M_PI) ;
    printf("Challis k      :  %12.2f arcsec\n",  kk* 180. * 3600./M_PI) ;

  } else if( cfg[METHOD].vl== POLARIS) {

/*     for( i=0 ; i < CONFIG_ENTRIES; i++) { */

/*       if( cfg[i].mthd_pola== TRUE) {   */
/* 	if(( cfg[i].mthd_pola== TRUE)) {  */
/* 	  printf( "Identifier: %32s, %f\n", cfg[i].id, cfg[i].vl) ; */
/* 	}	 */
/*       } */
/*     } */
    printf( "\n") ;

    pressure_water= pressure_h2o( cfg[TEMPERATURE].vl) ;
    pressure_total= pressure_tot( cfg[HEIGHT].vl) ;
    airpressure   = pressure_total- pressure_water ;



    printf( "Pressure total :  %12.2f hPa\n", pressure_total) ;
    printf( "Dry air        :  %12.2f hPa\n", airpressure) ;
    printf( "Water          :  %12.2f hPa\n", pressure_water) ;

    d_sv= d_s( airpressure, cfg[TEMPERATURE].vl) ;
    d_wv= d_w( pressure_water, cfg[TEMPERATURE].vl) ;


    n= refractive_index( lambda, d_sv, d_wv) ;

    //n= 1. ;
    printf( "Refractive Ind.:         %12.9f\n", n) ;

    /* Coordinates of Polaris */


    cfg[RIGHTASCENSION].vl= POLARIS_RA;
    cfg[DECLINATION].vl=  POLARIS_DEC;
    tau0= local_hour_angle(year, month, day, time_utc, cfg[RIGHTASCENSION].vl, cfg[OBS_LONGITUDE].vl) ;

    printf( "UTC time       :  %12.2f seconds\n", time_utc) ;
    printf( "Local hourangle:  %12.2f degree\n", tau0/M_PI * 180.) ;

/*     printf( "dtau1          :  %15.5f rad\n", dtau1) ; */
/*     printf( "dtau2          :  %15.5f rad\n", dtau2) ; */

    RY0p =  refraction_Yp( tau0,         cfg[DECLINATION].vl, cfg[OBS_LATITUDE].vl, n) ;

    Y1p= units_radian(cfg[RESOLUTION].vl/3600./180. * M_PI, cfg[DY1].vl)  ;

    Y1= Y1p-  (RY1p - RY0p) ;

   
    printf("Polaris     Y1 :  %12.2f arcsec\n", Y1 * 180. * 3600./M_PI) ;

    polaris_lambda= M_PI/2. -  cfg[DECLINATION].vl;

    printf("Polaris lambda :  %12.2f arcsec\n", polaris_lambda * 180. * 3600./M_PI) ;



    polaris_h=   tau0; // that's correct this way
    if( polaris_h < 0) {

      polaris_h = 2. * M_PI + polaris_h ;
    }

    printf("Polaris h      :  %12.2f deg\n", polaris_h * 180./M_PI) ;

    AA= King_A( polaris_lambda, polaris_h, cfg[OBS_LATITUDE].vl) ;
    kk= King_k( polaris_lambda, polaris_h) ;

    printf("\nPolaris A      :  %12.2f arcsec\n",  AA * 3600. * 180. /M_PI) ;
    printf("Polaris k      :  %12.2f arcsec\n",  kk* 180. * 3600./M_PI) ;

  }

  if(( fabs(AA)* 3600. * 180. /M_PI/cfg[RESOLUTION].vl < 10.) &&( fabs(kk)* 3600. * 180. /M_PI/cfg[RESOLUTION].vl < 10.)) {

    printf("\nYour are closer to the pole than Roald Amundsen was\n") ;

  }


  printf("\nMove the north end of the hour axis    ") ;
  printf("%6.2f", fabs(AA)* 3600. * 180. /M_PI/cfg[RESOLUTION].vl) ;

  if( AA >0) {
    printf(" units to the EAST\n") ;
  } else {
    printf(" units to the WEST\n") ;

  }

  printf("with star at declination %6.2f degree ", cfg[DECLINATION_M].vl * 180./M_PI) ; 

  printf("%6.2f", fabs(dAA(AA, cfg[OBS_LATITUDE].vl, cfg[DECLINATION_M].vl)) * 3600. * 180. /M_PI/cfg[RESOLUTION].vl) ;


  if( AA >0) {

    printf(" units to the EAST\n") ;
  } else {

    printf(" units to the WEST\n") ;
  }

  if( kk >0) {

    printf("\nLower ") ;
  } else {

    printf("\nIncrease ") ;
  }

  printf("elevation of the hour axis       %6.2f units\n\n", fabs(kk)* 3600. * 180. /M_PI/cfg[RESOLUTION].vl) ;
  
  printf("Remember\n") ;
  
  printf(" a refractor or a two mirror telescope simply rotates the field by 180 degree.\n") ;

  if( cfg[METHOD].vl== RAMBAUT) {

    double dist= 3600. * 180./M_PI * (n- 1) * tan( M_PI/2- cfg[OBS_LATITUDE].vl) ;
    printf(" the current apparent (\"refracted\") pole is %4.1f arcsec above the true pole.\n", dist) ;
  }
  printf("\n") ;


  return(0) ;
}
